package lcsb.mapviewer.api.configuration;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.view.ConfigurationView;

@RestController
public class ConfigurationController extends BaseController {
	@Autowired
	private ConfigurationRestImpl configurationController;

	@RequestMapping(value = "/configuration/getAllValues", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<ConfigurationView> getAllValues(@RequestParam(value = "token") String token) throws SecurityException {
		return configurationController.getAllValues(token);
	}

	@RequestMapping(value = "/configuration/getImageFormats", method = { RequestMethod.GET, RequestMethod.POST },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getImageFormats(@RequestParam(value = "token") String token) throws SecurityException {
		return configurationController.getImageFormats(token);
	}

	@RequestMapping(value = "/configuration/getModelFormats", method = { RequestMethod.GET, RequestMethod.POST },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getModelFormats(@RequestParam(value = "token") String token) throws SecurityException {
		return configurationController.getModelFormats(token);
	}

	@RequestMapping(value = "/configuration/", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> getOverlayTypes(@RequestParam(value = "token") String token) throws SecurityException {
		Map<String, Object> result = new HashMap<>();
		result.put("options", configurationController.getAllValues(token));
		result.put("imageFormats", configurationController.getImageFormats(token));
		result.put("modelFormats", configurationController.getModelFormats(token));
		result.put("overlayTypes", configurationController.getOverlayTypes(token));
		return result;
	}

	/**
	 * @return the configurationController
	 * @see #configurationController
	 */
	public ConfigurationRestImpl getConfigurationController() {
		return configurationController;
	}

	/**
	 * @param configurationController
	 *          the configurationController to set
	 * @see #configurationController
	 */
	public void setConfigurationController(ConfigurationRestImpl configurationController) {
		this.configurationController = configurationController;
	}

}