package lcsb.mapviewer.api.configuration;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestParam;

import lcsb.mapviewer.common.Pair;
import lcsb.mapviewer.converter.IConverter;
import lcsb.mapviewer.converter.graphics.AbstractImageGenerator;
import lcsb.mapviewer.converter.graphics.ImageGenerators;
import lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser;
import lcsb.mapviewer.converter.model.sbgnml.SbgnmlXmlConverter;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IConfigurationService;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.utils.data.ColorSchemaType;
import lcsb.mapviewer.services.view.ConfigurationView;

@Transactional(value = "txManager")
public class ConfigurationRestImpl {

	@Autowired
	private IUserService					userService;

	@Autowired
	private IConfigurationService	configurationService;

	public List<ConfigurationView> getAllValues(@RequestParam(value = "token") String token) throws SecurityException {
		userService.getToken(token);
		return configurationService.getAllValues();
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the configurationService
	 * @see #configurationService
	 */
	public IConfigurationService getConfigurationService() {
		return configurationService;
	}

	/**
	 * @param configurationService
	 *          the configurationService to set
	 * @see #configurationService
	 */
	public void setConfigurationService(IConfigurationService configurationService) {
		this.configurationService = configurationService;
	}

	public List<Map<String, Object>> getImageFormats(String token) throws SecurityException {
		userService.getToken(token);

		List<Map<String, Object>> result = new ArrayList<>();
		List<Pair<String, Class<? extends AbstractImageGenerator>>> imageGenerators = new ImageGenerators().getAvailableImageGenerators();

		for (Pair<String, Class<? extends AbstractImageGenerator>> element : imageGenerators) {
			Map<String, Object> row = new HashMap<>();
			row.put("name", element.getLeft());
			row.put("handler", element.getRight().getCanonicalName());
			result.add(row);
		}
		return result;
	}

	public List<Map<String, Object>> getModelFormats(String token) throws SecurityException {
		userService.getToken(token);
		List<IConverter> converters = new ArrayList<>();
		converters.add(new CellDesignerXmlParser());
		converters.add(new SbgnmlXmlConverter());

		List<Map<String, Object>> result = new ArrayList<>();

		for (IConverter converter : converters) {
			Map<String, Object> row = new HashMap<>();
			row.put("name", converter.getCommonName());
			row.put("handler", converter.getClass().getCanonicalName());
			result.add(row);
		}
		return result;
	}

	public List<Map<String, Object>> getOverlayTypes(String token) throws SecurityException {
		userService.getToken(token);
		List<Map<String, Object>> result = new ArrayList<>();
		for (ColorSchemaType type : ColorSchemaType.values()) {
			Map<String, Object> map = new HashMap<>();
			map.put("name", type.name());
			result.add(map);
		}
		return result;
	}

}
