package lcsb.mapviewer.api.projects;

import java.io.IOException;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.commands.CommandExecutionException;
import lcsb.mapviewer.converter.ConverterException;
import lcsb.mapviewer.model.cache.FileEntry;
import lcsb.mapviewer.model.map.InconsistentModelException;
import lcsb.mapviewer.model.map.layout.InvalidColorSchemaException;
import lcsb.mapviewer.services.SecurityException;

@RestController
public class ProjectController extends BaseController {
	@Autowired
	private ProjectRestImpl projectController;

	@RequestMapping(value = "/projects/{projectId}", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public ProjectMetaData getMetaData(//
			@PathVariable(value = "projectId") String projectId, //
			@RequestParam(value = "token") String token //
	) throws SecurityException {
		return projectController.getMetaData(projectId, token);
	}

	@RequestMapping(value = "/projects/{projectId}:downloadSource", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public ResponseEntity<byte[]> getProjectSource(//
			@RequestParam(value = "token") String token, //
			@PathVariable(value = "projectId") String projectId //
	) throws SecurityException, QueryException {

		FileEntry file = projectController.getSource(token, projectId);
		MediaType type = MediaType.TEXT_PLAIN;
		if (file.getOriginalFileName().endsWith("xml")) {
			type = MediaType.APPLICATION_XML;
		} else if (file.getOriginalFileName().endsWith("zip")) {
			type = MediaType.APPLICATION_OCTET_STREAM;
		}
		return ResponseEntity
				.ok().contentLength(file.getFileContent().length).contentType(type).header("Content-Disposition", "attachment; filename=" + file.getOriginalFileName())
				.body(file.getFileContent());
	}

	@RequestMapping(value = "/projects/{projectId}/models/{modelId}:downloadImage", method = { RequestMethod.GET },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public ResponseEntity<byte[]> getModelAsImage(//
			@RequestParam(value = "token") String token, //
			@PathVariable(value = "projectId") String projectId, //
			@PathVariable(value = "modelId") String modelId, //
			@RequestParam(value = "handlerClass") String handlerClass, //
			@RequestParam(value = "backgroundOverlayId", defaultValue = "") String backgroundOverlayId, //
			@RequestParam(value = "overlayIds", defaultValue = "") String overlayIds, //
			@RequestParam(value = "zoomLevel", defaultValue = "") String zoomLevel, //
			@RequestParam(value = "polygonString", defaultValue = "") String polygonString//
	) throws SecurityException, QueryException, IOException, InvalidColorSchemaException, CommandExecutionException {

		FileEntry file = projectController.getModelAsImage(token, projectId, modelId, handlerClass, backgroundOverlayId, overlayIds, zoomLevel, polygonString);
		MediaType type = MediaType.APPLICATION_OCTET_STREAM;
		return ResponseEntity
				.ok().contentLength(file.getFileContent().length).contentType(type).header("Content-Disposition", "attachment; filename=" + file.getOriginalFileName())
				.body(file.getFileContent());
	}

	@RequestMapping(value = "/projects/{projectId}/models/{modelId}:downloadModel", method = { RequestMethod.GET },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public ResponseEntity<byte[]> getModelAsModelFile(//
			@RequestParam(value = "token") String token, //
			@PathVariable(value = "projectId") String projectId, //
			@PathVariable(value = "modelId") String modelId, //
			@RequestParam(value = "handlerClass") String handlerClass, //
			@RequestParam(value = "backgroundOverlayId", defaultValue = "") String backgroundOverlayId, //
			@RequestParam(value = "overlayIds", defaultValue = "") String overlayIds, //
			@RequestParam(value = "zoomLevel", defaultValue = "") String zoomLevel, //
			@RequestParam(value = "polygonString") String polygonString//
	) throws SecurityException, QueryException, IOException, InvalidColorSchemaException, CommandExecutionException, ConverterException,
			InconsistentModelException {

		FileEntry file = projectController.getModelAsModelFile(token, projectId, modelId, handlerClass, backgroundOverlayId, overlayIds, zoomLevel, polygonString);
		MediaType type = MediaType.APPLICATION_OCTET_STREAM;
		return ResponseEntity
				.ok().contentLength(file.getFileContent().length).contentType(type).header("Content-Disposition", "attachment; filename=" + file.getOriginalFileName())
				.body(file.getFileContent());
	}

}