package lcsb.mapviewer.api.users;

import java.io.IOException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.Cookie;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.ObjectNotFoundException;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.interfaces.IUserService;
import lcsb.mapviewer.services.view.AuthenticationToken;

@RestController
public class UserController extends BaseController {
	Logger							 logger	= Logger.getLogger(UserController.class);

	@Autowired
	private IUserService userService;

	@Autowired
	private UserRestImpl userRest;

	@RequestMapping(value = "/doLogin", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> login(//
			@RequestParam(value = "login", defaultValue = Configuration.ANONYMOUS_LOGIN) String login, //
			@RequestParam(value = "password", required = false) String password, //
			HttpServletResponse response //
	) throws SecurityException, IOException {
		AuthenticationToken token = userService.login(login, password);
		if (token == null) {
			throw new SecurityException("Invalid credentials");
		} else {
			Map<String, Object> result = new HashMap<>();
			final Boolean useSecureCookie = false;
			final int expiryTime = (int) (token.getExpires().getTimeInMillis() - Calendar.getInstance().getTimeInMillis()) / 1000; // 24h
																																																														 // in
																																																														 // seconds
			final String cookiePath = "/";

			Cookie cookie = new Cookie("MINERVA_AUTH_TOKEN", token.getId());

			cookie.setSecure(useSecureCookie);
			cookie.setMaxAge(expiryTime);
			cookie.setPath(cookiePath);

			response.addCookie(cookie);
			response.getWriter().write("{\"info\":\"Login successful. TOKEN returned as a cookie\"}");
			response.getWriter().flush();
			response.getWriter().close();
			return result;
		}
	}

	@RequestMapping(value = "/users/{login}", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> getUser(//
			@RequestParam(value = "token", required = false) String token, //
			@PathVariable(value = "login") String login, //
			@RequestParam(value = "columns", defaultValue = "") String columns//
	) throws SecurityException, ObjectNotFoundException {
		return userRest.getUser(token, login, columns);
	}

	@RequestMapping(value = "/doLogout", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, String> logout(@RequestParam(value = "token", required = false) String token) throws SecurityException {
		userService.logout(token);
		Map<String, String> response = new HashMap<>();
		response.put("status", "OK");
		return response;
	}

	/**
	 * @return the userService
	 * @see #userService
	 */
	public IUserService getUserService() {
		return userService;
	}

	/**
	 * @param userService
	 *          the userService to set
	 * @see #userService
	 */
	public void setUserService(IUserService userService) {
		this.userService = userService;
	}

	/**
	 * @return the userRest
	 * @see #userRest
	 */
	public UserRestImpl getUserRest() {
		return userRest;
	}

	/**
	 * @param userRest
	 *          the userRest to set
	 * @see #userRest
	 */
	public void setUserRest(UserRestImpl userRest) {
		this.userRest = userRest;
	}
}